from lcyframe.libs.nsq_route import NsqTask, ReadNsq
from lcyframe import BaseModel

@NsqTask(topic="test",  # 订阅同一个topic的不同channel的消费者，均可收到消息
         channel="channel")
class ExampleEvent(ReadNsq, BaseModel):

    @classmethod
    def on_create_user(cls, **msg):
        print(msg, "channel 1")
        return True     # 必须返回True False None之一



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import time
import jwt
from jwt.exceptions import (
    DecodeError,
    ExpiredSignatureError,
    ImmatureSignatureError,
    InvalidAudienceError,
    InvalidIssuedAtError,
    InvalidIssuerError,
    MissingRequiredClaimError,
)
HS256 = "HS256"
default_headers = {"alg": HS256}

class JwtToken(object):

    def __init__(self, message, secret, exp=3600, algorithm=None, headers=None):
        self.secret = secret    # key
        self.exp = exp
        self.payload = {"message": message}
        self.algorithm = algorithm or HS256
        self.headers = headers or default_headers

    def get_token(self):
        return self.encode()

    def encode(self):
        return jwt.encode(self._get_playload(self.payload), self.secret, self.algorithm, self.headers)

    def decode(self, token):
        return jwt.decode(token, self.secret, self.algorithm)

    def is_expire(self, token):
        try:
            self.decode(token)
        except ExpiredSignatureError as e:
            return True
        else:
            return False

    def is_validate(self, token):
        try:
            payload = jwt.decode(token, self.secret, self.algorithm)
            return True
        except Exception as e:
            return False

    def refresh_token(self, token):
        try:
            self.payload = self.decode(token)
            return self.encode()
        except Exception as e:
            return False

    def _get_playload(self, payload):
        assert isinstance(payload, dict)

        expire_time = int(time.time()) + self.exp
        payload.update({"exp": expire_time})

        return payload

if __name__ == '__main__':
    t = JwtToken(54887111, "f465ca164f0e", 3600)
    token = t.encode()
    print(token)
    message, exp = t.decode(token).values()
    print(message, exp)

    print(t.is_expire(token))
    print(t.is_validate(token))
    print(t.refresh_token(token))


# coding=utf-8
import sys, os
from bson.objectid import ObjectId
from base import BaseModel
from lcyframe import yaml2py
from lcyframe import ServicesFindet
from lcyframe.libs.singleton import CeleryCon
from lcyframe.celery_server import CeleryWorker
from context import InitContext

# args = [item.split("=")[-1] for item in sys.argv if item.startswith("--config")]
sys.argv = sys.argv[:1]
config = InitContext.get_context()

tasks = CeleryWorker(**config)
yaml2py.impmodule(BaseModel, "model")       # 换为载入model父类，切worker的函数继承了model父类，所以可以使用所有的model子类
ServicesFindet(tasks, BaseModel)(config)    # 获取全站所有连接库，赋值给BaseModel，继承了BaseModel的worker内，含有mqtt生产者连接，可以使用生产者创建新的任务
app = CeleryCon._app                        # 命令行模式运行：celery -A celery_start worker --loglevel=info --pool=eventlet --concurrency=10 -Qapp_queue


if __name__ == "__main__":                  # Debug调试模式启动
    tasks.app = app
    tasks.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.admin_schema import AdminSchema

class AdminModel(BaseModel, AdminSchema):

    @classmethod
    def get_admin(cls, admin_id, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        d = cls.find_one_by_oid(admin_id)
        return cls._parse_data(d)


    @classmethod
    def get_admin_by_spec(cls, spec, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        d = cls.find_one(spec)
        return cls._parse_data(d)


    @classmethod
    def get_admin_list_by_last_id(cls, last_id, count, **kwargs):
        """
        前端列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, last_id = cls.find_list_by_last_id(spec,
                                                      count,
                                                      sort=[("create_at", -1), ],
                                                      fields=False,
                                                      last_id_field=False)
        return [cls._parse_data(d) for d in data_list if d], last_id

    @classmethod
    def get_admin_list_by_page(cls, page, count, **kwargs):
        """
        后台列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, pages = cls.find_list_by_page(spec,
                                                 page,
                                                 count,
                                                 sort=[("create_at", -1), ],
                                                 fields=False)
        return [cls._parse_data(d) for d in data_list if d], pages

    @classmethod
    def create_admin(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pwd = kwargs.pop("pass_word")

        if not pwd or not kwargs["user_name"]:
            return -1

        if cls.find_one({"nick_name": kwargs["nick_name"]}):
            return - 2

        docs = {}
        docs["gid"] = kwargs["gid"]
        docs["uid"] = cls.IdGeneratorModel.gen_uid_id()
        docs["salt"] = cls.utils.gen_salt()
        docs["pass_word"] = cls.utils.gen_salt_pwd(pwd, docs["salt"])
        docs.update(kwargs)
        cls.insert(docs)
        return docs["uid"]

    @classmethod
    def modify_admin(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        spec = {"uid": kwargs.pop("uid")}
        update_docs = {}
        for k, v in kwargs.items():
            if k not in vars(cls.UserSchema()):
                continue

            # 改密码
            if k == "pass_word" and kwargs["pass_word"]:
                update_docs[k] = cls.utils.gen_salt_pwd(kwargs["pass_word"], kwargs["salt"])
            # 改用户组
            elif k == "gid":
                update_docs[k] = v
                update_docs["permission"] = cls.UserSchema.gid_permission[v]
            else:
                update_docs[k] = v

        return cls.update(spec, update_docs)

    @classmethod
    def delete_admin(cls, admin_id, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        return cls.update({"_id": admin_id}, {"state": -1})

    @classmethod
    def _parse_data(cls, d, **kwargs):
        """
        组装单条数据
        :return:
        :rtype:
        """
        if not d:
            return {}

        d["admin_id"] = d.pop("_id", "")

        return d





#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
import sys
import importlib
# from multiprocessing import Process
import logging

import socketserver


root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)
from context import InitContext
from base import BaseModel
import re
from lcyframe import ServicesFindet

logger = logging.getLogger(__name__)


def load_model(cls, model_dir=None):
    """
    import model class obj global
    warning: the class obj cant not start whit "__"
    :return:
    """
    model_dir = model_dir or cls.model_dir
    sys.path.append(model_dir)
    for m in [x for x in os.listdir(model_dir) if re.findall('[A-Za-z]\w+_model\.py$', x)]:
        m = m[:-3]
        if m == "__init__":
            continue
        for attr_name, value in importlib.import_module(m).__dict__.items():
            if attr_name.startswith("__"):
                continue

            if not hasattr(BaseModel, attr_name):
                setattr(BaseModel, attr_name, value)
                logging.debug("model [%s] register success!" % attr_name)

class Server(object):
    def __init__(self, **kwargs):
        self.ROOT = kwargs["ROOT"]
        self.socket_servers = kwargs['socket_servers']
        self.register()
        self.config = kwargs

    def register(self):
        pass

    def start(self):
        for server_conf in self.socket_servers:
            port = server_conf['port']
            mod, func = server_conf['path'].rsplit('.', 1)
            handler = getattr(importlib.import_module(mod), func)
            handler.config = self.config
            ServicesFindet(handler, BaseModel)(handler.config)
            load_model(handler, "model")
            socketserver.TCPServer.allow_reuse_address = True
            s = socketserver.ThreadingTCPServer(('0.0.0.0', port), handler)
            logger.warning("%s is start" % server_conf)
            s.serve_forever()
            # p = Process(target=s.serve_forever)
            # p.start()


if __name__ == '__main__':
    config = InitContext.get_context()
    server = Server(**config)
    server.start()



from lcyframe import MqRoute
from lcyframe import BaseModel
from tornado.gen import Return, coroutine

@MqRoute.tasks
class UserEvent(BaseModel):
    @classmethod
    @coroutine
    def update_header(cls, **msg):
        print(msg)
        raise Return(True)



# -*- coding:utf-8 -*-
from lcyframe.libs import Copyright

def get_handler_init(path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))

""" % (Copyright.file_pre)

    import_str = """
# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue
    logging.debug("handler [%s] register success!" % hander_file)
    __import__(model_name, globals(), locals(), [model_name], -1)
"""

    return auto_str + import_str


def get_docs_agree():
    """

    :return:
    """
    auto_str = """
## 一、约定说明
### 1.1 参数说明

**参数位置：本框架规定了所有请求时，参数所配置的位置说明：**
* query: 参数位置在？后面：http://www.domain.com/path?{name}={value}
* form-data: 参数位置在body里,以multipart/form-data方式提交。通常表现为json格式
* www-form: 参数位置在body里,以multipart/x-www-form-urlencoded方式提交。通常为表单键值对格式
* file: 参数位置在body里,以multipart/form-data方式提交，常用户上传数据
* raw：不建议
* binary：不建议

**参数值类型：在设置schema.yml时，指定type的值允许是以下几种之一**
* int: 整形
* float: 浮点型
* str: 字符串
* json: json串
* file: 上传文件

### 1.2 headers
API请求时，需要提供请求头

headers：
```
{
    "uid": 用户uid, int,  注册成功、登录成功后返回
    "token": 授权码, str, 注册成功、登录成功后返回
    "locale": "cn",
    "lang": "",
    "version": "", 默认 1.0
    "bundle": "",
    "platform": 0 IOS, 1 Android
    "os": "",
    "device_id": ""
}
```
### 1.3 API 返回结果
所有http_status响应值为200的请求，仅代表本次HTTP请求响应成功。
返回以下2种结果之一：
* 当error=0时，代表应用服务器处理成功，客户端可以正常处理data
* 当error!=0时，代表应用服务器处理失败，error即为对应的异常码。
* msg：字段为本次请求的提示信息
* data：为本次请求返回的数据

result:
```
成功：{"error": 0, "msg": msg, "data": {}}
失败：{"error": 1, "msg": msg, "data": {}}
```

### 1.4 双向校验码seq生成
部分防刷接口需要提供参数seq，生成方法如下

salt：写死在客户端
```
例如：
params={"nickname": "阿刘", "age": "111", "timestamp": 1473761433}
上述2个参数value 均为 utf-8 编码, 根据key的字典顺序排序 键和值之间用"="连接 键值对之间用"&"连接,
生成待校验的字符串string, string 与 salt 字符串 拼接,计算校验码：
seq = md5(salt + "nickname=阿刘&age=111&timestamp=1473761433")
注：凡需要传seq的接口，必须同时提供请求时间戳参数：timestamp
```

### 1.5 翻页码规则：仅支持查看下一页，不支持跳页
参数名：last_id，由服务器返回。客户端可以在请求列表时，回传给服务端，
```
    1、当不提供该参数时，代表查看第一页
    2、当服务端返回last_id=-1时，代表没有下一页了
```

### 1.6 域名配置
api：http://api.domain.com
admin 接口：http://xx.domain.com
docs文档：http://docs.domain.com
"""
    return auto_str


def get_mq_work_init():
    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
    """ % Copyright.file_pre

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+_worker\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    logging.debug("register mq workers [%s.py] success!" % model_name)
    __import__(model_name, globals(), locals(), [model_name], -1)
    """

    return auto_str + import_str


def get_nsq_producer_init(root_path, produce_path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
from importlib import import_module
from lcyframe.libs.singleton import NsqCon
nsq = NsqCon.get_connection()

dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
produce_path = "%s"
""" % (Copyright.file_pre, produce_path)

    import_str = """
# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue

    model_name = produce_path.strip("/").replace("/", ".") + "." + model_name
    objs = __import__(model_name, fromlist=["*"])
    for p in dir(objs):
        if p.startswith("__"):
            continue
            
        # if not p.lower().startswith("publish"):
            #logging.debug("nsq producer [%s] load fail. it must been start with 'publish'" % p)
        #    continue
        
        if not hasattr(getattr(objs, p), "topic"):
            continue
        
        logging.debug("nsq producer [%s] register success!" % p)
        setattr(nsq, p, getattr(objs, p)())
"""

    return auto_str + import_str


def get_nsq_work_init():
    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
    """ % Copyright.file_pre

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+_worker\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    logging.debug("register nsq workers [%s.py] success!" % model_name)
    __import__(model_name, globals(), locals(), [model_name], -1)

    """
    return auto_str + import_str


def get_mqtt_producer_init(root_path, produce_path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
from importlib import import_module
from lcyframe.libs.singleton import MqttCon
mqtt = MqttCon.get_connection()

dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
produce_path = "%s"
""" % (Copyright.file_pre, produce_path)

    import_str = """
# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue

    model_name = produce_path.strip("/").replace("/", ".") + "." + model_name
    objs = __import__(model_name, fromlist=["*"])
    # objs = import_module(model_name)  # 重名py文件会被覆盖
    for p in dir(objs):
        if p.startswith("__"):
            continue
            
        # if not p.lower().startswith("publish"):
            #logging.debug("mqtt producer [%s] load fail. it must been start with 'publish'" % p)
        #    continue
        
        if not hasattr(getattr(objs, p), "topic"):
            continue
            
        logging.debug("mqtt producer [%s] register success!" % p)
        setattr(mqtt, p, getattr(objs, p)())
"""

    return auto_str + import_str


def get_mqtt_work_init():
    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
    """ % Copyright.file_pre

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+_worker\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    logging.debug("register mqtt workers [%s.py] success!" % model_name)
    __import__(model_name, globals(), locals(), [model_name], -1)

    """
    return auto_str + import_str

def get_celery_works_init(root_path, workers_path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
from lcyframe.libs.singleton import CeleryCon
from lcyframe.libs.celery_route import Events
app = CeleryCon._app

dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
workers_path = "%s"
""" % (Copyright.file_pre, workers_path)

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue

    model_name = workers_path.strip("/").replace("/", ".") + "." + model_name
    objs = __import__(model_name, fromlist=["*"])
    module_name = objs.__name__.split(".")[-1]

    setattr(Events, module_name, objs)

    for p in dir(objs):
        if p.startswith("__"):
            continue

        if type(getattr(objs, p)) == type:
            if not hasattr(getattr(objs, p), "queue"):
                continue

            if hasattr(app, p) and p != 'BaseEvent':
                raise Exception("exists same attribute：%s" % p)

        # logging.debug("celery task [%s] register success!" % p)
        setattr(Events, p, getattr(objs, p))

    setattr(app, Events.__name__, Events)

"""

    return auto_str + import_str


# coding=utf-8
from magic_numbers import *


PIL_INSTALLED = False
try:
    from PIL import Image

    PIL_INSTALLED = True
except ImportError:
    raise

def check_name(name):
    try:
        name = name.encode('utf-8')
    except UnicodeDecodeErroras e:
        pass
    name = name.replace(' ', '') \
        .replace('.', '') \
        .replace('-', '') \
        .replace('_', '') \
        .replace('（', '(') \
        .replace('）', ')') \
        .replace(',', '') \
        .replace('+', '')

    try:
        name = name.decode('utf-8')
    except UnicodeDecodeErroras e:
        pass

    return name


def get_store_goods_img_url(filename, size=1):
    '''
    获取商城商品图片url
    :param filename:
    :param size: 图片大小预留
    :return:
    '''
    return "%sstore/%s" % (QINIU_IMAGE, filename)


def get_backimg_url(filename, size=1):
    if size == RAW:
        return "%sback/%s" % (QINIU_IMAGE, filename)
    return "%sback/%s-%s" % (QINIU_IMAGE, filename, THUMB_MAP[size]) if filename else ""


def get_avatar_url(filename, size=1):
    if not filename:
        return ""
    if size == RAW:
        return "%savatar/%s" % (QINIU_IMAGE, filename)
    return "%savatar/%s-%s" % (QINIU_IMAGE, filename, THUMB_MAP[size])


def get_post_url(filename, size=1):
    if not filename:
        return ""
    # if size == RAW:
    #     return "%simage/%s" % (QINIU_IMAGE, filename)
    return "%simage/%s-%s" % (QINIU_IMAGE, filename, THUMB_MAP[size])


def get_audio_url(filename):
    if not filename:
        return ""
    return "%saudio/%s" % (QINIU_AUDIO, filename)


def get_video_url(filename):
    if not filename:
        return ""
    return "%svideo/%s" % (QINIU_MEDIA, filename)


def get_cover_url(filename, size=1):
    if not filename:
        return ""
    if isinstance(filename, list):
        for i in filename[::-1]:
            if i:
                filename = i
                break
    # if size == RAW:
    #     return "%scover/%s" % (QINIU_IMAGE, filename)
    return "%scover/%s-%s" % (QINIU_IMAGE, filename, THUMB_MAP[size])


def mget_cover_url(filename, size=1):
    pics = []
    if not filename:
        return pics
    if isinstance(filename, list):
        for i in filename:
            if not i:
                continue
            if size == RAW:
                # i = "%scover/%s" % (QINIU_IMAGE, i)
                i = "%scover/%s-%s" % (QINIU_IMAGE, i, THUMB_MAP[size])
            else:
                i = "%scover/%s-%s" % (QINIU_IMAGE, i, THUMB_MAP[size])
            pics.append(i)
    return pics


def get_bgt_url(filename):
    if not filename:
        return ""

    return "%sbgt/%s" % (QINIU_BGT, filename)


def get_version_url(filename):
    if not filename:
        return ""

    return "%spack/%s" % (QINIU_BGT, filename)


def get_post_media_url(type, media):
    if type == 0:
        return get_audio_url(media)
    return get_video_url(media)


def get_works_thumb_url(thumb, t=WORKS):
    if thumb == '':
        thumb = 'moren@3x.png'
    elif thumb == [''] or thumb == []:
        thumb = ['moren@3x.png']

    if not isinstance(thumb, list):
        return [get_cover_url(thumb, t)]
    return [get_cover_url(i, t) for i in thumb]


def get_ad_url(filename, size=6):
    if not filename:
        return ""
    if isinstance(filename, list):
        for i in filename[::-1]:
            if i:
                filename = i
                break
    # if size == RAW:
    #     return "%scover/%s" % (QINIU_IMAGE, filename)
    return "%sad/%s-%s" % (QINIU_IMAGE, filename, THUMB_MAP[size])


def get_replay_url(filename):
    if not filename:
        return ""

    return "%sreplay/%s" % (QINIU_REPLAY, filename)


def get_replay_img_url(filename):
    if not filename:
        return ""

    return "%simage/%s" % (QINIU_REPLAY, filename)


def get_replay_ppt_url(filename):
    if not filename:
        return ""

    return "%sppt/%s" % (QINIU_REPLAY, filename)


def get_replay_lyric_url(filename):
    if not filename:
        return ""

    return "%slyric/%s" % (QINIU_REPLAY, filename)


# -*- coding: utf-8 -*-
from celery import shared_task
import random, time
from lcyframe.libs.celery_route import BaseTask
from lcyframe.libs.celery_route import BaseEvent

@shared_task(ignore_result=False)  # 普通函数装饰为 celery task
def save_image2(x):
    """
    seleep时间随机
    任务多线程并发执行，谁先执行完毕，谁先输出结果
    :param x:
    :return:
    """
    n = round(random.random(), 2)
    time.sleep(n)

    # model.UserModel.get_xxx()
    return x, n


class UserEvent2(BaseEvent):
    @staticmethod
    @shared_task(ignore_result=False)  # 普通函数装饰为 celery task
    def register(x):
        """
        seleep时间随机
        任务多线程并发执行，谁先执行完毕，谁先输出结果
        :use self.application.celery.DefaultEvent.add.delay(111)
        :param x:
        :return:

        """
        n = round(random.random(), 2)
        time.sleep(n)

        return x, n



# coding=utf-8
import logging
import hmac
import base64
import random
import time
from pathlib import Path
from datetime import timedelta, datetime, date
from string import ascii_letters
import zlib
import hashlib, string
import json
from bson.objectid import ObjectId
import operator
import re
import traceback

_chars = string.printable[:87] + '_' + string.printable[90:95]
to_ObjectId = lambda a: ObjectId(a) if type(a) != ObjectId else a
to_python = lambda s: json.loads(s)
to_json = lambda obj: json.dumps(obj, ensure_ascii=False, sort_keys=True)
fix_path = lambda path: Path(path).as_posix()
traceback = traceback

def random_int(length=6):
    """生成随机的int 长度为length"""
    return ("%0" + str(length) + "d") % random.randint(int("1" + "0" * (length - 1)), int("9" * length))

def gen_random_str(length=6, chars=_chars):
    return ''.join(random.choice(chars) for i in range(length))

def gen_random_sint(length=6):
    """
    获取字符串加数字的随机值
    :param length:
    :return:
    """
    return "".join(random.choice(string.hexdigits) for i in range(length))

def random_string(length=6):
    """生成随机字符串"""
    return ''.join(random.choice(ascii_letters) for i in range(length))

def gen_hmac_key():
    """随机生成长度32位密文"""
    s = str(ObjectId())
    k = gen_random_str()
    key = hmac.HMAC(k, s).hexdigest()
    return key

def enbase64(s):
    """
    编码
    :param s:
    :return:
    """
    if type(s) == bytes:
        return base64.b64encode(s)
    else:
        s = s.encode('utf-8')
        return base64.b64encode(s).decode('utf-8')

def debase64(s):
    """
    解码
    :param s:
    :return:
    """
    bytes_types = (bytes, bytearray)
    return base64.b64decode(s) if isinstance(s, bytes_types) else base64.b64decode(s).decode()


class TypeConvert(object):
    """类型转换类, 处理参数"""
    MAP = {int: int,
           float: float,
           bool: bool,
           str: str}

    STR2TYPE = {"int": int,
                "integer": int,
                "string": str,
                "str": str,
                "bool": bool,
                "float": float,
                "list": list,
                "dict": dict,
                "json": json.loads}

    @classmethod
    def apply(cls, obj, raw):
        try:
            tp = type(obj)
            if tp in TypeConvert.MAP:
                return TypeConvert.MAP[tp](raw)
            return obj(raw)
        except Exception as e:
            logging.error("in TypeConvert.apply %s, obj: %s, raw: %s" % (e, obj, raw))
            return None

    @classmethod
    def convert_params(cls, _type, value):
        if _type in ["int", "integer"] and not value:
            value = 0
        try:
            tp = TypeConvert.STR2TYPE[_type]
            return tp(value)
        except Exception as e:
            raise e

def calculate_age(ts):
    """计算年龄"""
    if ts == -1:
        return -1
    born = datetime.fromtimestamp(ts)
    today = date.today()
    return today.year - born.year - ((today.month, today.day) < (born.month, born.day))


def now():
    return int(time.time())

def oid_to_date(oid):
    return int_to_date_string(int(str(oid)[:8], 16))


def int_to_date_string(ts, fm=False):
    fm = fm if fm else "%Y-%m-%d %H:%M"
    try:
        if not ts:
            ts = 0
        return datetime.fromtimestamp(ts).strftime(fm)
    except:
        return datetime.fromtimestamp(time.time()).strftime(fm)

def str2timestamp(date_string, fm=False):
    fm = fm if fm else "%Y-%m-%d"
    return int(time.mktime(time.strptime(date_string, fm)))
    # return int(time.mktime(datetime.strptime(date_string, format).timetuple()))

def timestamp2str(ts, fm=False):
    return int_to_date_string(ts, fm)

def get_yesterday_midnight():
    # 获取昨天的午夜时间戳
    return get_today_midnight() - 86400

def get_today_midnight():
    # 获取今天的午夜时间戳
    now = int(time.time())
    return now - now % 86400 - 3600 * 8 - 86400

def get_today_lasttime():
    # 获取今天最后一秒时间戳
    now = int(time.time())
    return now - now % 86400 - 3600 * 8 + 24 * 3600 - 1


def get_delta_day(day=1):
    """
    获取n天后的时间
    :param day:
    :return:
    """

    now = datetime.now()

    # 当前日期
    now_day = now.strftime('%Y-%m-%d %H:%M:%S')

    # n天后
    delta_day = (now + timedelta(days=int(day))).strftime("%Y-%m-%d %H:%M:%S")
    return delta_day

def get_ts_from_object(s):
    if len(s) == 24:
        return int(s[:8], 16)
    return 0

def compress_obj(dict_obj, compress=True):
    """反序列化dict对象"""
    dict_obj = {"$_key_$": dict_obj} if not isinstance(dict_obj, dict) else dict_obj
    if compress:
        return zlib.compress(to_json(dict_obj))
    return to_json(dict_obj)

def uncompress_obj(binary_string, compress=True):
    """反序列化dict对象"""
    if compress:
         dict_obj = to_python(zlib.decompress(binary_string))
    else:
        dict_obj = to_python(binary_string)

    if "$_key_$" in dict_obj:
        return dict_obj["$_key_$"]
    else:
        return dict_obj

def get_mod(uid, mod=10):
    return int(uid) % mod

def gen_salt(len=6):
    return ''.join(random.sample(string.ascii_letters + string.digits, len))

def gen_salt_pwd(salt, pwd):
      return hashlib.md5((str(salt) + str(pwd)).encode("utf-8")).hexdigest()

def md5(s):
    return hashlib.md5(s).hexdigest()


def version_cmp(version1, version2):
    """比较系统版本号
    v1 > v2 1
    v1 = v2 0
    v1 < v2 -1
    v1: 用户使用的版本
    v2：最新上线的版本
    """

    def normalize(v):
        return [int(x) for x in re.sub(r'(\.0+)*$', '', v).split(".")]

    return operator.gt(normalize(version2), normalize(version1))

def _find_option_with_arg(argv, short_opts=None, long_opts=None):
    """Search argv for options specifying short and longopt alternatives.

    Returns:
        str: value for option found
    Raises:
        KeyError: if option not found.

    Example：
        config_name = _find_option_with_arg(short_opts="-F", long_opts="--config")
    """
    for i, arg in enumerate(argv):
        if arg.startswith('-'):
            if long_opts and arg.startswith('--'):
                name, sep, val = arg.partition('=')
                if name in long_opts:
                    return val if sep else argv[i + 1]
            if short_opts and arg in short_opts:
                return argv[i + 1]
    raise KeyError('|'.join(short_opts or [] + long_opts or []))

def check2json(data):
    if isinstance(data, (list, tuple)):
        for index, item in enumerate(data):
            data[index] = check2json(item)
        return data
    elif isinstance(data, dict):
        for key, value in data.items():
            data[key] = check2json(value)
        return data
    elif isinstance(data, ObjectId):
        return str(data)
    else:
        return data




# coding=utf-8
import os
from json import loads, dumps
import logging
import time
from bson.objectid import ObjectId
from traceback import format_exc
from paho.mqtt import client as mqtt, publish as _publish
from lcyframe.libs.crypto import encrypt, decrypt
from lcyframe.libs import utils


class MqttTask(object):
    """订阅队列"""
    _workers = {}

    def __init__(self, topic, qos=2):
        self.topic = topic
        self.qos = qos

    def __call__(self, _handler):
        setattr(_handler, "topic", self.topic)
        setattr(_handler, "qos", self.qos)
        self._workers[_handler.__name__] = _handler
        return _handler

    @classmethod
    def get_workers(cls, ROOT, workers_path):
        if not ROOT:
            raise Exception("the project dir path must been give， and None.")

        if not isinstance(workers_path, list):
            workers_path = [workers_path]

        if workers_path is None:
            raise Exception("mqtt workers_path is not allow empty")

        if not cls._workers:
            for work in workers_path:
                for root, dirs, files in os.walk(os.path.join(ROOT, work)):
                    for file in files:
                        if file.startswith("__"):
                            continue
                        if file.endswith(".pyc"):
                            continue
                        if not file.endswith(".py"):
                            continue
                        model_name = root.replace(ROOT, "").lstrip("/").replace("/", ".") + "." + file.rstrip(".py")
                        __import__(model_name, globals(), locals(), [model_name], 0)
                        logging.debug("register mqtt workers [%s.py] success!" % model_name)
        return cls._workers


class ConnectMqtt(object):
    _mqttc = None

    def __init__(self, **config):
        self.mqtt_config = config
        self.mqtt_host = self.mqtt_config.get('host', '127.0.0.1')
        self.mqtt_port = self.mqtt_config.get('port', 1883)
        self.client_id_prefix = self.mqtt_config.get('client_id_prefix', '')
        self.client_id_name = self.mqtt_config.get('client_id_name', str(ObjectId()))
        self.transport = self.mqtt_config.get("transport", "tcp")
        self.keepalive = self.mqtt_config.get("keepalive", 60)

        self.authorization = self.mqtt_config.get("authorization", {})
        self.username = self.authorization.get('username', '')
        self.password = str(self.authorization.get('password', ''))

        self.certificate = self.mqtt_config.get("certificate", {})
        self.cafile = self.certificate.get("cafile")
        self.certfile = self.certificate.get("certfile")
        self.keyfile = self.certificate.get("keyfile")

        self.msg_pwd = str(self.mqtt_config.get('msg_pwd', ''))
        self.encrypt_model = int(self.mqtt_config.get("encrypt_model", 1))

        self.client_role = self.mqtt_config.get("client_role", "producer")    # 链接角色，生产者：producer；消费者：worker
        self.client_id = self.client_id_prefix + self.client_id_name if self.client_id_prefix else self.client_id_name

    @property
    def mqttc(self):
        if ConnectMqtt._mqttc is None:
            _mqttc = mqtt.Client(client_id=self.client_id,
                                 protocol=mqtt.MQTTv311,
                                 transport=self.transport)  # transport="tcp/websockets" 取决于你的配置文件protocol用了那种协议

            pwd = self.mqtt_config.get("authorization", {}).get("use")
            cert = self.mqtt_config.get("certificate", {}).get("use")
            if pwd == cert == True:
                raise Exception("禁止同时开启密码、证书认证")

            if pwd:
                _mqttc.username_pw_set(self.username, password=self.password)

            elif cert:
                _mqttc.tls_set(self.cafile, self.certfile, self.keyfile)

            _mqttc.msg_pwd = self.msg_pwd
            _mqttc.encrypt_model = self.encrypt_model

            _mqttc.connect(self.mqtt_host, self.mqtt_port, self.keepalive)
            if self.client_role == "producer":
                _mqttc.loop_start()
            else:
                # _mqttc.loop_forever()
                pass

            ConnectMqtt._mqttc = _mqttc
        return ConnectMqtt._mqttc


class ReadMqtt(ConnectMqtt):
    """
    消费者：当接收新消息后调用该方法
    """

    def __init__(self, **config):
        super(ReadMqtt, self).__init__(**config)

    def on_log(self, client, obj, level, string):
        """
        实现日志能力
        :param mqttc:
        :param obj:
        :param level:
        :param string:
        :return:
        """
        pass

    def connect(self):
        """

        :return:
        """
        self.mqttc.connect(self.mqtt_host, self.mqtt_port)

    def on_message(self, client, obj, message):
        """
        接收到消息后调用
        message.topic
        message.payload
        message.qos

        :param mqttc:
        :param obj:
        :param msg:
        :return:
        """
        try:
            message = decrypt(message.payload, self.msg_pwd, self.encrypt_model)
        except Exception as e:
            message = message.payload

        try:
            msg = loads(message)
        except Exception as e:
            msg = message

        # msg["client_id"] = client._client_id

        try:
            event = msg.get("event", "")
            topic = msg.get("topic", "")  # "tpoic1","*"
        except:
            logging.warning("message:" + msg)
            logging.warning("event name can not None！")
            return

        for work_name, work in self.workers.items():
            if hasattr(work, event):
                try:
                    logging.debug("时间: %s, topic: %s, event: %s" % (utils.int_to_date_string(utils.now(), "%Y-%m-%d %H:%M:%S"), topic.encode("u8"), event.encode("u8")))

                    if work.topic == topic or work.topic == "#":
                        getattr(work, event)(**msg)
                    else:
                        continue
                except:
                    logging.warning(format_exc())
                    logging.warning(msg)

    def on_connect(self, client, obj, flags, rc):
        """
        订阅topic，连接成功后调用, 支持模糊匹配topic层级
        :param mqttc:
        :param obj:
        :param flags:
        :param rc:
        :return:
        """
        if not self.workers:
            logging.warning("has not work handler.you must load first.")
            return

        topic_list = []
        for name, handler in self.workers.items():
            topic_list.append((getattr(handler, "topic"), getattr(handler, "qos", 2)))

        client.subscribe(topic_list)

        logging.debug("mqtt client connect success. %s" % client._client_id)

    def on_publish(self, client, userdata, mid):
        """
        发送成功后调用
        :param c:
        :param obj:
        :param mid:
        :return:
        """
        logging.debug("mqtt client publish success. %s" % client._client_id)

    def on_disconnect(self, client, userdata, rc):
        """
        客户端断开连接后调用
        :param c:
        :param obj:
        :param mid:
        :return:
        """
        logging.debug("mqtt client disconnect. %s" % client._client_id)

    def publish(self, topic, payload=None, qos=2, retain=False):
        """
        发送消息, 较慢
        :param topic:
        :param payload:
        :param qos:
        :param retain:
        :return:
        """
        if topic is None or len(topic) == 0:
            raise ValueError('Invalid topic.')

        payload = encrypt(payload, self.msg_pwd, self.encrypt_model)
        _publish.single(topic,
                        payload=payload,
                        qos=qos,
                        retain=retain,
                        keepalive=self.keepalive,
                        hostname=self.mqtt_host,
                        port=self.mqtt_port,
                        protocol=mqtt.MQTTv311,
                        auth={'username': self.username, "password": self.password},
                        transport=self.transport
                        )

    def start(self):
        """

        :return:
        """
        self.mqttc.on_message = self.on_message
        self.mqttc.on_log = self.on_log
        self.mqttc.on_connect = self.on_connect
        self.mqttc.on_publish = self.on_publish
        self.mqttc.on_disconnect = self.on_disconnect
        self.mqttc.loop_forever()


class WriteMqtt(object):
    """
    生产者：向已订阅的topic里发送消息
    topic, qos 需要在你的业务逻辑里实现赋值
    """

    _mqtt = ConnectMqtt.mqttc

    def __getattr__(self, name):
        if hasattr(self.__class__, '__events__'):
            if name not in self.__class__.__events__:
                raise Exception("Event '%s' is not exists in your Producer Class" % name)

        if name not in self.__dict__:
            if not hasattr(self, "qos"):
                self.qos = 2
            self.__dict__[name] = pub = AgentMqtt(self._mqtt, name, self.topic, self.qos)
            return pub
        else:
            return self.__dict__[name]


class AgentMqtt:
    def __init__(self, mqtt, event, topic, qos=2):
        self.event = event
        self.topic = topic
        self.mqtt = mqtt
        self.qos = qos

    def __repr__(self):
        return "event '%s'" % self.event

    def __call__(self, kwargs):
        payload = {}
        topic = kwargs.get("topic", "")
        qos = kwargs.get("qos", self.qos)
        retain = kwargs.get("retain", False)

        if kwargs:
            payload["event"] = self.event  # message_handler name by Consumer
            payload["topic"] = self.topic  #
            payload["ts"] = utils.now()
            payload.update(kwargs)
        
        payload = encrypt(dumps(payload), self.mqtt.msg_pwd, self.mqtt.encrypt_model) if len(payload) > 1 else None
        self.mqtt.publish(topic or self.topic, payload, int(qos), retain)




import functools
import logging
import math
import os

from crontab import CronTab
from tornado.ioloop import PeriodicCallback


log_crontab = logging.getLogger("tornado-crontab.crontab")
FORMAT_LOG_CRONTAB = " ".join(["tornado-crontab[%(pid)d]:",
                               "(%(user)s)",
                               "FUNC",
                               "(%(funcname)s",
                               "%(args)s %(kwargs)s)"])


class CronTabCallback(PeriodicCallback):

    def __init__(self, callback, schedule, io_loop=None):
        self.__crontab = CronTab(schedule)
        super(CronTabCallback, self).__init__(
            callback, self._calc_callbacktime(), io_loop)
        self.pid = os.getpid()
        if os.name == "nt":
            self.user = os.environ.get("USERNAME")
        else:
            import pwd
            self.user = pwd.getpwuid(os.geteuid()).pw_name

    def _calc_callbacktime(self, now=None):
        return math.ceil(self.__crontab.next(now)) * 1000.0

    def _get_func_spec(self):

        _args = []
        _kwargs = {}

        def _get_func(_func):

            if not isinstance(_func, functools.partial):
                return _func

            for _arg in reversed(_func.args):
                _args.insert(0, _arg)

            if _func.keywords:
                _kwargs.update(_func.keywords)

            return _get_func(_func.func)

        _func = _get_func(self.callback)
        return _func, _args, _kwargs

    def _logging(self, level):

        if self._running and log_crontab.isEnabledFor(level):

            _func, _args, _kwargs = self._get_func_spec()

            log_crontab.log(level,
                            FORMAT_LOG_CRONTAB % dict(pid=self.pid,
                                                      user=self.user,
                                                      funcname=_func.__name__,
                                                      args=_args,
                                                      kwargs=_kwargs))

    def _run(self):

        self._logging(logging.INFO)

        try:
            PeriodicCallback._run(self)

        finally:

            self._logging(logging.DEBUG)

    def _schedule_next(self):
        self.callback_time = self._calc_callbacktime()
        super(CronTabCallback, self)._schedule_next()


def crontab(schedule, io_loop=None):

    def receive_func(func):

        @functools.wraps(func)
        def wrapper(*args, **kwargs):

            _func = functools.partial(func, *args, **kwargs)
            CronTabCallback(_func, schedule, io_loop).start()

        return wrapper
    return receive_func


# coding=utf-8
from cStringIO import StringIO
from tornado import gen
from sevencow import Cow
from base64 import urlsafe_b64encode
from magic_numbers import *
import hashlib

"""
This is a use example, you can copy to your project and rewrite.
"""

def check_name(name):
    try:
        name = name.encode('utf-8')
    except UnicodeDecodeErroras e:
        pass
    name = name.replace(' ', '') \
        .replace('.', '') \
        .replace('-', '') \
        .replace('_', '') \
        .replace('（', '(') \
        .replace('）', ')') \
        .replace(',', '') \
        .replace('+', '')

    try:
        name = name.decode('utf-8')
    except UnicodeDecodeErroras e:
        pass

    return name


PIL_INSTALLED = False
try:
    from PIL import Image

    PIL_INSTALLED = True
except ImportError:
    raise


class Qiniu(Cow):
    def __init__(self, **kwargs):
        super(Qiniu, self).__init__(**kwargs)

    def generate_filename(self, data, name=None):

        if name:
            filename = check_name(name)
        else:
            # filename = random_string(10)
            filename = hashlib.md5(data).hexdigest()

        return filename

    @gen.coroutine
    def save_store_img(self, data):
        '''
        上传商城详情和主图图片
        :param data:
        :return:
        '''
        bucket, prefix = BUCKET_MAP[FILE_TYPE_STORE]
        task, filenames = [], []

        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))
        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_avatar(self, data, bucket, prefix):
        task, filenames = [], []

        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket,
                                 self.make_thumb(i, THUMB_SIZE[MAX]) if PIL_INSTALLED else i["body"],
                                 filename="/".join([prefix, filename]),
                                 content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_local_avatar(self, filePath):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_AVATAR]
        task, filenames = [], []
        with open(filePath, 'rb', encoding='utf-8') as f:
            data = f.read()
        filename = self.generate_filename(data)
        filenames.append(filename)
        task.append(self.put(bucket, data, filename="/".join([prefix, filename]), content_type='image/jpeg'))
        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_backimg(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_BACKIMG]

        task, filenames = [], []

        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_post(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_POST]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_cover(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_COVER]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(data=i['body'])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_local_cover(self, filePath):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_COVER]
        task, filenames = [], []
        with open(filePath, 'rb', encoding='utf-8') as f:
            data = f.read()
        filename = self.generate_filename(data)
        filenames.append(filename)
        task.append(self.put(bucket, data, filename="/".join([prefix, filename]), content_type='image/jpeg'))
        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_ad(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_AD]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(data=i['body'])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_audio(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_AUDIO]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            if i["content_type"] != 'audio/mp3':
                i["content_type"] = 'audio/mp3'
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_video(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_VIDEO]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_pack(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_PACK]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    @gen.coroutine
    def save_replay(self, data):
        bucket, prefix = BUCKET_MAP[FILE_TYPE_REPLAY]
        task, filenames = [], []
        for i in data:
            filename = self.generate_filename(i["body"])
            filenames.append(filename)
            task.append(self.put(bucket, i["body"], filename="/".join([prefix, filename]), content_type=i["content_type"]))

        yield task
        raise gen.Return(filenames)

    def make_thumb(self, upload_file, size=(240, 240), info=False):
        """生成缩略图"""

        try:
            thumb = Image.open(StringIO(upload_file["body"]))
        except IOError as e:
            raise e

        width, height = thumb.size
        if width > size[0] or height > size[1]:
            thumb.thumbnail(size, Image.ANTIALIAS)

        output = StringIO()
        thumb.save(output, thumb.format, quality=80)
        if not info:
            return output.getvalue()
        width, height = thumb.size
        return output.getvalue(), width, height

    def make_crop_thumb(self, upload_file, size=(128, 128), info=False):
        """生成带裁剪的缩略图"""
        im = Image.open(StringIO(upload_file["body"]))
        width, height = im.size
        if width <= height:
            offset = int((height - width) * 0.25)
            box = (0, offset, width, offset + width)
            region = im.crop(box)
        else:
            offset = int((width - height) / 2)
            box = (offset, 0, height + offset, height)
            region = im.crop(box)

        croped_width, croped_height = region.size

        if croped_width > size[0] or croped_height > size[1]:
            region.thumbnail(size, Image.ANTIALIAS)

        output = StringIO()
        region.save(output, im.format, quality=80)

        if not info:
            return output.getvalue()

        width, height = region.size
        return output.getvalue(), width, height

    def get_upload_token(self, filt_type, filename):
        """上传token"""
        bucket, prefix = BUCKET_MAP[int(filt_type)]
        token = self.generate_upload_token('%s:%s/%s' % (bucket, prefix, filename))
        return token, '%s/%s' % (prefix, filename)

    def get_upload_video_token(self, filename, cut_img, offset=1, watermark_media=None):
        """watermark_media: 上传策略
        1、上传后是否加水印
        2、播放时是否有限播放带水印的视频
        3、视频截图尺寸
        4、预处理使用的队列
        5、回调地址
        预处理规则：avthumb/mp4/vb/1.25m/wmImage/%s|saveas/%s;vframe/jpg/offset/%s/w/1024/h/950|saveas/%s
        解释：上传视频成功后 1、转为mp4且加上水印，然后保存到指定地方；2、取视频第n秒的截图，缩放为1024*950尺寸的jpg图片，保存在iguitar-image空间，保存名称为cover/filename
            1 和 2 顺序不能颠倒
        """
        bucket, prefix = BUCKET_MAP[FILE_TYPE_VIDEO]
        cover_bucket, cover_prefix = BUCKET_MAP[FILE_TYPE_COVER]    

        ops = dict()
        avthumbOps = ''
        if watermark_media.get('wmImage', True):
            avthumbOps = 'avthumb/mp4/vb/1.25m/wmImage/%s/rotate/auto|saveas/%s' % (urlsafe_b64encode(watermark_media["wmImageUrl"]),
                                                                        urlsafe_b64encode('%s:%s/%s' % (bucket, prefix, filename + '_wmImage')))
            ops['persistentNotifyUrl'] = watermark_media["NotifyUrl"]


        if watermark_media.get('cutjpg_w') and watermark_media.get('cutjpg_h'):
            vframeOps = 'vframe/jpg/offset/%s/w/%s/h/%s|saveas/%s' % (offset, watermark_media.get('cutjpg_w'), watermark_media.get('cutjpg_h'),
                                                                     urlsafe_b64encode('%s:%s/%s' % (cover_bucket, cover_prefix, cut_img)))
        else:
            # rotate/auto 会导致图片或视频翻转，原理不清
            # vframeOps = 'vframe/jpg/offset/%s/rotate/auto|saveas/%s' % (get_second(duration), urlsafe_b64encode('%s:%s/%s' % (cover_bucket, cover_prefix, cut_img)))
            vframeOps = 'vframe/jpg/offset/%s|saveas/%s' % (offset, urlsafe_b64encode('%s:%s/%s' % (cover_bucket, cover_prefix, cut_img)))

        if avthumbOps:
            persistentOps = avthumbOps + ';' + vframeOps
        else:
            persistentOps = vframeOps

        ops['persistentOps'] = persistentOps
        ops['persistentPipeline'] = watermark_media["Pipeline"]

        scope = '%s:%s/%s' % (bucket, prefix, filename)
        token = self.generate_upload_token(scope, ops=ops)
        return token, '%s/%s' % (prefix, filename)




#!/usr/bin/env python
# -*- coding:utf-8 -*-

import logging
import os
import signal
import platform
import tornado
import tornado.web
from tornado.httpserver import HTTPServer
from tornado.ioloop import IOLoop, PeriodicCallback
from tornado.options import options
from tornado.httpclient import AsyncHTTPClient
from tornado.gen import Task
from .libs import Copyright
from .libs import yaml2py, utils
from .libs.route import route
from .libs.docs.generate_docs import GenerateDocs
from .libs.services import ServicesFindet
from .base import BaseHandler, BaseModel
from .test.docshandler import DocsHandler


class App(tornado.web.Application):
    def __init__(self, port=None, **kwargs):
        """
        A Tornado web frame .
        :param args:
        :param kwargs:
                    debug: default True
                    template_path:
                    static_path:
                    cookie_secret:
                    handler_dir: the handler of web api
                    max_buffer_size:
                    logging_level: debug
                    parse_command_line: view the logging of request. default True
                    db: mongo_db obj
                    redis: redis obj
                    ssdb: ssdb obj
                    solr:
                    task: List. PeriodicCallback handler obj [(obj, callback_time), ...]
        """
        kwargs["ROOT"] = utils.fix_path(kwargs["ROOT"])
        os.environ.app_config = kwargs
        self.kwargs = kwargs
        self.ROOT = utils.fix_path(kwargs["ROOT"])
        self.kwargs["Copyright"] = Copyright
        self.uri = kwargs.get("host") or kwargs["wsgi"].get("host", "127.0.0.1")
        self.port = port or kwargs["wsgi"].get("port", kwargs.get("port", 6678))
        self.debug = kwargs["wsgi"].get("debug", True)
        self.max_buffer_size = int(kwargs["wsgi"].get("max_buffer_size", 10 * 1024 * 1024 * 1024))

        self.template_path = self.get_path(kwargs.get("template_path", "template"))
        self.static_path = self.get_path(kwargs.get("static_path", "static"))
        self.handler_dir = self.get_path(kwargs.get("handler_dir", "handler"))
        self.model_dir = self.get_path(kwargs.get("model_dir", "model"))
        self.api_schema_dir = self.get_path(kwargs.get("api_schema_dir", "api_schema"))
        self.errors_dir = self.get_path(kwargs.get("errors_dir", "utils"))
        self.api_docs = kwargs.get("api_docs", {})
        self.docs_dir = self.get_path(self.api_docs.get("docs_dir", "docs"))
        self.jinja2_path_dir = kwargs.get("jinja2_dir", "jinja2")

        self.cookie_secret = kwargs.get("cookie_secret", "f465ca164f0e")
        self.settings = dict(debug=self.debug,
                             template_path=self.template_path,
                             static_path=self.static_path,
                             cookie_secret=self.cookie_secret)

        self.log_config = self.kwargs.get("logging_config", {})
        self.init_log()
        self.load_api_schema()
        self.create_base()
        self.create_handler()
        self.create_model()
        self.impmodule()
        self.handlers = route.get_routes(self.ROOT, self.handler_dir)
        self.handlers.append((r'/docs', DocsHandler))
        self.handlers.append((r'/.*', BaseHandler))
        if not self.handlers:
            raise Exception("no api handler has been register.")

        self.task = kwargs.get("task", [])  # you must give the obj list like this: [(func, 1000)]

        tornado.web.Application.__init__(self, self.handlers, **self.settings)

    def check_path(self, path):
        if not os.path.exists(path):
            raise IOError("[Errno 2] No such file or directory: %s" % path)
        else:
            return True

    def get_path(self, name):
        """

        :param p:
        :return:
        """
        return utils.fix_path(os.path.join(self.ROOT, name.lstrip("/")))

    def load_api_schema(self):
        """
        read schema from .yml
        :return:
        """
        assert self.check_path(self.api_schema_dir)
        self.api_schema_mp = yaml2py.load_api_schema(self.api_schema_dir)

    def create_base(self):
        s = os.path.join(self.ROOT, "base.py")
        if not os.path.exists(s) and self.kwargs.get("auto_generate_base", True):
            yaml2py.generate_basepy(self.ROOT + "/base.py", self.jinja2_path_dir)

    def create_handler(self):
        """
        create xxx_handler.py
        :return:
        """
        if not self.kwargs.get("auto_generate_py", True):
            return

        handler_init = os.path.join(self.handler_dir, "__init__.py")
        if not os.path.exists(self.handler_dir) or not os.path.exists(handler_init):
            if not os.path.exists(self.handler_dir):
                os.mkdir(self.handler_dir)

        yaml2py.generate_resource_handler(self.handler_dir, self.model_dir, self.api_schema_mp, self.jinja2_path_dir)

    def create_model(self):
        """
        create mode of model
        :return:
        """
        if not self.kwargs.get("auto_generate_py", True):
            return

        self.model_schema_dir = self.model_dir + "/schema"
        model_init = os.path.join(self.model_dir, "__init__.py")

        if not os.path.exists(self.model_dir) or not os.path.exists(model_init):
            if not os.path.exists(self.model_dir):
                os.mkdir(self.model_dir)

            if not os.path.exists(model_init):
                with open(os.path.join(model_init), "w", encoding='utf-8') as p:
                    p.write(Copyright.file_pre)

            schema_init = os.path.join(self.model_schema_dir, "__init__.py")
            if not os.path.exists(self.model_schema_dir):
                os.mkdir(self.model_schema_dir)

            if not os.path.exists(schema_init):
                with open(schema_init, "w", encoding='utf-8') as p:
                    p.write(Copyright.file_pre)

        yaml2py.generate_resource_model(self.model_dir, self.api_schema_mp, self.jinja2_path_dir)

        schema_init = os.path.join(self.model_schema_dir, "__init__.py")
        if not os.path.exists(schema_init):
            with open(schema_init, "w", encoding='utf-8') as p:
                p.write(Copyright.file_pre)

        yaml2py.generate_resource_model_schema(self.model_schema_dir, self.api_schema_mp, self.jinja2_path_dir)

    def impmodule(self, model_dir=None):
        model_dir = model_dir or self.model_dir
        return yaml2py.impmodule(BaseModel, model_dir)

    def init_setattr(self):
        ServicesFindet(BaseHandler, BaseModel)(self.kwargs)
        setattr(BaseModel, "uri", self.uri)
        setattr(self, "app_config", self.kwargs)
        setattr(BaseHandler, "api_schema", self.api_schema_dir)
        setattr(BaseHandler, "app_config", self.kwargs)
        setattr(BaseModel, "api_schema", self.api_schema_dir)
        setattr(BaseModel, "app_config", self.kwargs)
        setattr(BaseHandler, "request_url", self.request_url)
        setattr(BaseModel, "request_url", self.request_url)

    def init_log(self):
        """
        ###############################################
        # 日志模块配置
        # 使用循环日志文件保存
        ###############################################
        open: true  # 是否开启日志
        # warn 控制台只打印异常请求记录堆栈，debug 打印所有请求记录
        level: warn
        #日志保存地址, 非必填，留空则不保存到文件
        file: "logs/"
        #日志文件名字,不提供时，采用端口号作为名称。port.log
        name: app.log         # 默认格式.log
        #循环日志的个数，app.log,app.log1,app.log2
        nums: 10
        #日志文件的大小，500M
        size: 524288000 # 0.5 * 1024 * 1024
        stderr: True
        :return:
        """
        if self.log_config.get("save"):
            tornado.options.options.log_file_max_size = self.log_config.get("size", 524288000)
            tornado.options.options.log_file_num_backups = self.log_config.get("nums", 10)
            file = self.log_config.get("file")
            if not os.path.abspath(file):
                raise Exception("log path must is abdpath.")

            if not os.path.exists(file):
                os.makedirs(file)

            path = "%s/%s" % (file, self.log_config.get("name", str(self.kwargs["wsgi"]["port"])))
            tornado.options.options.log_file_prefix = path

        tornado.options.options.logging = self.log_config.get('level', "debug")
        tornado.options.parse_command_line()

    def periodic_callback(self):
        for obj, t in self.task:
            PeriodicCallback(obj, t).start()

    def request_url(self, url):
        http_client = AsyncHTTPClient()
        response = yield Task(http_client.fetch, url)
        if response.error:
            raise Exception(response.error)
        return response

    def start(self):
        self.init_setattr()
        tornado.options.define(name="app_config", default=self.kwargs, help='App server config', type=str)
        http_server = tornado.httpserver.HTTPServer(self, max_buffer_size=self.max_buffer_size, xheaders=True)
        http_server.bind(self.port)
        logging.warning("Server Config file: " + self.kwargs["config_name"])
        logging.warning("Server ROOT path: " + self.ROOT)
        logging.warning('Server Running on http://%s:%s' % (self.uri, self.port))
        if platform.system() == "Windows":
            import asyncio
            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
        http_server.start()
        ioloop = IOLoop.instance()
        self.periodic_callback()
        GenerateDocs.start(self)
        def signal_handler(sig, frame):
            ioloop.add_callback(shutdown)
        def shutdown():
            logging.warning('Stopping http server')
            http_server.stop()
            ioloop.add_callback(ioloop.stop)
        signal.signal(signal.SIGINT, signal_handler)
        signal.signal(signal.SIGTERM, signal_handler)
        ioloop.start()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Generated by nelson


"""Reusable functional retry patterns.

Certain kinds of operations are not always reliable, such as accessing
a network (the network may be down), or communicating with a remote
service (the service may be busy). These operations can fail with
"transient" errors. Given enough time, transient errors may be resolved.
Therefore, the caller may wish to retry the operation several times in
the hope that it will eventually succeed. Callers often want to control
the number of retries, the total amount of time spent retrying, and other
properties of the retry operation. Taken together, these properties
are called a "retry policy". The functions exported by this module
enable a retry policy to be defined and then used to govern the retry
of any arbitrary asynchronous operation. Furthermore, the retry policy
and manager can be sub-classed if custom retry policies are desired.

Example Usage:

  # Create retry policy that retries 3 times, with no delay between attempts,
  # as long as the HTTP response has an error. Typically this policy would
  # be created once and reused throughout the application.
  http_retry_policy = RetryPolicy(max_tries=3,
                                  check_result=lambda resp: resp.Error)

  # Retry an asynchronous HTTP fetch operation using this policy.
  CallWithRetryAsync(http_retry_policy, client.fetch,
                     'http://www.google.com',
                     callback=handle_response)

  # Create retry policy that retries for up to 30 seconds, starting with
  # a retry interval of at least 1 second, and exponentially backing off
  # to at most 10 seconds between retries. Retry if the operation fails
  # with an exception.
  retry_policy = RetryPolicy(timeout=timedelta(seconds=30),
                             min_delay=timedelta(seconds=1),
                             max_delay=timedelta(seconds=10),
                             check_exception=lambda typ, val, tb: True)
  # or use check_exception=RetryPolicy.AlwaysRetryOnException
"""

__author__ = 'andy@emailscrubbed.com (Andy Kimball)'

import sys
import random
import functools
import time
import logging
import traceback

from datetime import timedelta
from tornado.ioloop import IOLoop
from tornado.stack_context import ExceptionStackContext
from functools import partial

def FormatFunctionCall(func, *args, **kwargs):
    """Format a function and its arguments in a human-readable way for logging."""
    while type(func) is partial:
        args = func.args + args
        if func.keywords:
            kwargs.update(func.keywords)
        func = func.func

    return "%s%s" % (func.__name__, FormatArguments(*args, **kwargs))


def FormatLogArgument(s):
    """Format "s" in a human-readable way for logging by truncating it
    to at most 256 characters.
    """
    MAX_LEN = 256
    if isinstance(s, unicode):
        s = s.encode('utf-8')
    else:
        s = str(s)
    if len(s) <= MAX_LEN:
        return s
    return '%s...[%d bytes truncated]' % (s[:MAX_LEN], len(s) - MAX_LEN)


def FormatArguments(*args, **kwargs):
    """Format function call arguments in a human-readable way for logging."""

    def _FormatArg(arg):
        return FormatLogArgument(arg)

    # Truncate arguments.
    args = [_FormatArg(arg) for arg in args]
    kwargs = {key: _FormatArg(value) for key, value in kwargs.items()}

    return "(args=%s, kwargs=%s)" % (args, kwargs)


class RetryPolicy(object):
    """Defines a group of properties that together govern whether and how
    an operation will be retried. The base class defines a number of common
    retry properties. It can handle two common asynchronous patterns:

      - Errors are never raised as exceptions, but instead passed to the
        callback function. Define the "check_result" argument in this case.
      - Errors are raised as exceptions, which are expected to be handled
        by a stack_context. Define the "check_exception" argument in this
        case.

    However, if additional customization is needed, then a custom retry
    policy can be created to work with CallWithRetryAsync. To do this,
    create a subclass of RetryPolicy as well as RetryManager, and
    re-implement the constructor(s), CreateManager, and/or DoRetry.
    """

    def __init__(self, max_tries=sys.maxint, timeout=timedelta.max, min_delay=timedelta(seconds=0),
                 max_delay=timedelta.max, check_result=None, check_exception=None):
        """Initialize a default instance of the RetryPolicy, choosing among
        the following properties:

        max_tries (int)
          Maximum number of tries that will be attempted.

        timeout (timedelta or int or float)
          If this amount of time is exceeded, then the operation will not be
          retried. This is only checked between attempts. If a number is
          provided, then it is interpreted as a number of seconds.

        min_delay (timedelta or int or float)
          Minimum delay between attempts. After the first attempt, at least
          this amount of time must pass before the second attempt will be
          made. For subsequent tries, the delay will exponentially increase,
          up to the limit specified in max_delay. If a number is provided,
          then it is interpreted as a number of seconds.

        max_delay (timedelta or int or float)
          Maximum delay between attempts. It is useful to cap this in order
          to guarantee that an attempt will be tried at a minimum frequency,
          even after exponential back-off. If a number is provided, then it
          is interpreted as a number of seconds.

        check_result (func(*callback_args, **callback_kwargs))
          When an asynchronous operation completes, this function is passed
          the arguments to the callback function. Therefore, its signature
          should match that of the callback passed to the retry-able function.
          The check_result function should return true if the operation has
          failed with a retry-able error, or false otherwise.

        check_exception (func(type, value, traceback))
          If the asynchronous function throws an Exception as this function is
          passed the parts of the exception. The check_exception should return
          true if the operation has failed with a retry-able error, or false
          otherwise.

        The exponential backoff algorithm helps to prevent "retry storms", in
        which many callers are repeatedly and insistently retrying the same
        operation. This behavior can compound any existing problem. In addition,
        a random factor is added to the backoff time in order to desynchronize
        attempts that may have been aligned.
        """
        self.max_tries = max_tries
        self.timeout = timeout if type(timeout) is timedelta else timedelta(seconds=timeout)
        self.min_delay = min_delay if type(min_delay) is timedelta else timedelta(seconds=min_delay)
        self.max_delay = max_delay if type(max_delay) is timedelta else timedelta(seconds=max_delay)
        self.check_result = check_result
        self.check_exception = check_exception

    def CreateManager(self):
        """Called by CallWithRetry in order to create a RetryManager which can
        track the progress of a particular operation. This method can be overridden
        if a custom retry policy is created.
        """
        return RetryManager(self)

    @staticmethod
    def AlwaysRetryOnException(type, value, traceback):
        """Static method to always retry on exceptions."""
        return True


class RetryManager(object):
    """For each kind of RetryPolicy, there should be a corresponding
    RetryManager which tracks the retry progress of a particular operation.
    The CallWithRetryAsync function will call CreateManager on the
    RetryPolicy instance in order to get a manager that it can use to
    track the progress of retries by calling the DoRetry method. The
    manager may track how many tries have been attempted, how much time
    has elapsed, etc.

    When the asynchronous operation has completed, CallWithRetryAsync
    will invoke the MaybeRetryOnResult function. If the asynchronous
    operation fails with an Exception as then CallWithRetryAsync will
    invoke the MaybeRetryOnException function. If the invoked function
    returns false, then no retry will be attempted. Instead, the original
    callback will be invoked (in case of MaybeRetryOnResult), or the
    exception will be re-raised (in case of MaybeRetryOnException).
    However, if the function returns true, then CallWithRetryAsync
    expects it to invoke "retry_func" once the retry should be attempted.
    The function should never block the calling thread, but it may
    perform a non-blocking wait before invoking "retry_func".
    """

    def __init__(self, retry_policy):
        """Create a RetryManager that is capable of tracking properties
        defined in the RetryPolicy base class. This involves tracking the
        number of tries attempted so far, along with whether the timeout
        deadline has been exceeded.
        """
        self.retry_policy = retry_policy
        self._num_tries = 0
        self._deadline = time.time() + retry_policy.timeout.total_seconds()
        self._delay = None

    def MaybeRetryOnResult(self, retry_func, *result_args, **result_kwargs):
        """This function is called by CallWithRetryAsync once the asynchronous
        operation has completed and has invoked its callback function. It
        returns true if a retry should be attempted.
        """

        def CheckRetry():
            """Retry should be attempted if the result inspector function exists and returns true."""
            return self.retry_policy.check_result and self.retry_policy.check_result(*result_args, **result_kwargs)

        def GetLoggingText():
            """Return text that will be logged if retry is necessary."""
            return '%s returned %s' % (FormatFunctionCall(retry_func),
                                       FormatArguments(*result_args, **result_kwargs))

        return self._MaybeRetry(retry_func, CheckRetry, GetLoggingText)

    def MaybeRetryOnException(self, retry_func, type, value, tb):
        """This function is called by CallWithRetryAsync if the asynchronous
        operation raises an exception. It returns true if a retry should be
        attempted.
        """

        def CheckRetry():
            """Retry should be attempted if the exception inspector function exists and returns true."""
            return self.retry_policy.check_exception and self.retry_policy.check_exception(type, value, tb)

        def GetLoggingText():
            """Return text that will be logged if retry is necessary."""
            return '%s raised exception %s' % (FormatFunctionCall(retry_func),
                                               traceback.format_exception(type, value, tb))

        return self._MaybeRetry(retry_func, CheckRetry, GetLoggingText)

    def _MaybeRetry(self, retry_func, check_retry_func, log_func):
        """Helper function that determines whether a retry should be attempted.
        A retry is only attempted if "check_func" returns true. The "log_func"
        is invoked if a retry is attempted in order to get text that shows
        the context of the retry, and which will be logged.
        """
        # Check whether max tries have been exceeded.
        self._num_tries += 1
        if self._num_tries >= self.retry_policy.max_tries:
            return False

        # Check whether timeout has been exceeded.
        if time.time() >= self._deadline:
            return False

        # Invoke caller-defined function that determines whether a retry-able error has occurred.
        if not check_retry_func():
            return False

        # Retry after delay.
        if not self._delay:
            self._delay = self.retry_policy.min_delay
        else:
            self._delay *= 2

        # Cap delay.
        if self._delay > self.retry_policy.max_delay:
            self._delay = self.retry_policy.max_delay

        # Add random factor to desynchronize retries, still capped by max delay.
        sleep_time = timedelta(seconds=(random.random() + 1) * self._delay.total_seconds())
        if sleep_time > self.retry_policy.max_delay:
            sleep_time = self.retry_policy.max_delay

        # Start asynchronous sleep and instruct caller to retry.
        logging.getLogger().warning('Retrying function after %.2f seconds: %s' %
                                    (sleep_time.total_seconds(), log_func()))

        if sleep_time.total_seconds() == 0:
            IOLoop.current().add_callback(retry_func)
        else:
            IOLoop.current().add_timeout(sleep_time, retry_func)

        return True


def CallWithRetryAsync(retry_policy, func, *args, **kwargs):
    """This is a higher-order function that wraps an arbitrary asynchronous
    function (plus its arguments) in order to add retry functionality. If the
    wrapped function completes with an error, then CallWithRetryAsync may call
    it again. Pass a "retry_policy" argument that derives from RetryPolicy in
    order to control the retry behavior. The retry policy determines whether
    the function completed with a retry-able error, and then decides how many
    times to retry, and how frequently to retry.
    """
    # Validate presence of named "callback" argument.
    inner_callback = kwargs.get('callback', None)
    assert 'callback' in kwargs, 'CallWithRetryAsync requires a named "callback" argument that is not None.'

    retry_manager = retry_policy.CreateManager()

    # Called when "func" is complete; checks whether to retry the call.
    def _OnCompletedCall(*callback_args, **callback_kwargs):
        """Called when the operation has completed. Determine whether to retry,
        based on the arguments to the callback.
        """
        retry_func = functools.partial(func, *args, **kwargs)
        if not retry_manager.MaybeRetryOnResult(retry_func, *callback_args, **callback_kwargs):
            # If the async operation completes successfully, don't want to retry if continuation code raises an exception.
            exception_context.check_retry = False
            inner_callback(*callback_args, **callback_kwargs)

    def _OnException(type, value, tb):
        """Called if the operation raises an exception. Determine whether to retry
        or re-raise the Exception as based on the exception details.
        """
        if exception_context.check_retry:
            print("in retry")
            retry_func = functools.partial(func, *args, **kwargs)
            return retry_manager.MaybeRetryOnException(retry_func, type, value, tb)

    # Replace the callback argument with a callback to _OnCompletedCall which will check for retry.
    kwargs['callback'] = _OnCompletedCall

    # Catch any exceptions in order to possibly retry in that case.
    exception_context = ExceptionStackContext(_OnException)
    exception_context.check_retry = True
    with exception_context:
        func(*args, **kwargs)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from base import BaseSchema
import time

class PermissionGroupSchema(BaseSchema):
    """
    permission
    """

    collection = "permission_group"

    def __init__(self):
        self._id = ObjectId()
        self.available = True   # ******
        self.permission_class = {}  # ******
        self.gid = 1    # ******ID
        self.name = "******2"
        self.pid = 1    # ******ID
        self.desc = ""  # ******
        self.create_at = int(time.time())
        self.update_at = int(time.time())

class PermissionClassSchema(BaseSchema):
    """
    permission
    """

    collection = "permission_class"

    def __init__(self):
        self._id = ObjectId()
        self.available = True   # ******
        self.bit_mp = {}  # ******bit
        self.key = 1    # ******ID
        self.name = "******"
        self.desc = ""  # ******
        self.create_at = int(time.time())
        self.update_at = int(time.time())



# -*- coding:utf-8 -*-
import json
import tornado.httpserver
import tornado.ioloop
from tornado.web import RequestHandler, Application
import tornado.options
from tornado.options import define, options
from tornado.escape import json_decode, json_encode
import random, time
# tasks.py
from celery import Celery



app = Celery('tasks', backend='redis://localhost:3379/0',
             broker='redis://localhost:3379/0')  # 配置好celery的backend和broker

from celery import Task

class DebugTask(Task):
    abstract = True

    def after_return(self, *args, **kwargs):
        print('Task returned: {0!r}'.format(self.request))

    def on_retry(self, exc, task_id, args, kwargs, einfo):
        print("on_failure(self, exc, task_id, args, kwargs, einfo)")

@app.task(base=DebugTask)
def add(x, y):
    return x + y

@app.task(bind=True, autoretry_for=(Exception,), retry_kwargs={'max_retries': 3, 'countdown': 5}, base=DebugTask)
def test_func(self):
    print(1111)
    raise Exception()


if __name__ == "__main__":
    # send_wx_text('a')
    print(add(1, 2))
    print(test_func())


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import sys, os
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)
from . import *
from lcyframe.libs import cprint

def upload():
    headers = {"uid": 100000}

    params = {

    }
    files = {"excel": open("Yijian_project/xxxxxxx.xlsx", "rb")}    # excel 是在yml定义的参数名

    return admin_login_and_send(
        methed="post",
        url="/upload",
        params=params,
        headers=headers,
        files=files
    )


if __name__ == "__main__":
    upload()




#!/usr/bin/env python
# -*- coding:utf-8 -*-
import sys, os
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)
from context import InitContext
from lcyframe.libs.singleton import MongoCon
from lcyframe.libs.cprint import cprint
from lcyframe.libs.JWT import JwtToken
import requests

config = InitContext.get_context()
db = MongoCon().get_database(**config["mongo_config"])
aes_secret = "******"
jwt_srcret = "******"
private_key = "-----BEGIN RSA PRIVATE KEY-----MIICYAIBAAKBgQDadqkuZj2A6gEW2HRKDgMX3XUfJp/NvA7ON2EUEJnBZQJ0Da8iFjqSnnu1fYHCBtDBC3m5b0xjBRQo91ZHYR9lHW5SAcWieoDWE/sYf6zsWiuJi8T7kum9oMyBAxjGhKD0HdGzYNN8+4f37wijbHR5mgFyzIdzmJPSXcUc+Zx4mQIDAQABAoGAako9ehPIbMZtjT33Jmi23I+lAmj6a1DKK08Kboa9KDkK/ogB46XJDbkKG31a+pnyS1pX+P7LvYmlC2F7JwRJjfSKGOAl34900o3rXK/55jbi+tUViruvEsZbJsM345FNSU4bf+Ct3ng1znrhrkuo76HUTw4FPzd3acb0AhHc7dkCRQDuSE9Gjq6WAO3w+1Fk6AzxVwTxiiLOVC5Ibc6wZWc3uwtncAmLm2S5TM6V/QCBuBDXHtzmppBVExbbHXHV7hiWsGsfDwI9AOq1GVjzKi5Dtr3dOpy7mhzX14TRGjS8gUZeQIBD0tmKLJKZEWkDA8rYMNLhA9IYMrOPb1+zMkXc1dRt1wJEGK4cp+43XwoRmxgswgrW7FhbBrmMCVrmwFG/Sr32Buu0rq4IDxG1SQwPibF+z/DErcNglfNCl802XNOb6tCSc2kqbzkCPGFIRwVTZLxgXLI9rDmimLIz1KS8dvw81ehw0JNZiV+ZoffxcgHwufWtvi7qDUdbuEgsv6EPCVtjuU2faQJFAIxj3I638xoF4AABzaVuxqBIMsacJOZhOH0Rbxo1oVVDkZ4uHa2GyoIAg7cDHzZqFV6GPa5qMa+Q0vH7GQjwwgJLU5WS-----END RSA PRIVATE KEY-----"
HEARDERS = {"appkey": ""}

def gen_token(uid, secret=jwt_srcret):
    return JwtToken(uid, secret).get_token()

def get_params(methed, docs):
    doc = docs[methed]
    params = {}

    if not doc:
        return params

    if isinstance(doc[0], dict):
        doc = doc[0].items()

    for k, v in doc:
        params[k] = v

    return params

def admin_login_and_send(methed, url, params, **kwargs):

    if "/" not in url:
        url = "/" + url

    global HEARDERS
    if "headers" not in kwargs:
        kwargs["headers"] = HEARDERS
    else:
        HEARDERS.update(kwargs["headers"])
        kwargs["headers"] = HEARDERS

    if "token" not in kwargs["headers"]:
        kwargs["headers"]["token"] = gen_token(kwargs["headers"].get("appkey")
                                               or kwargs["headers"].get("uid")
                                               or kwargs.get("agentid"))

    for k, v in kwargs["headers"].items():
        if isinstance(v, int):
            kwargs["headers"][k] = str(v)

    _url = config["wsgi"]["host"] #

    if "http" not in config["wsgi"]["host"]:
        _url = "http://" + _url + ":" + str(config["wsgi"]["port"])
    elif "127.0.0.1" in _url:
        _url = config["wsgi"]["host"] + ":" + str(config["wsgi"]["port"])
    _url += url


    if methed == "delete":
        kwargs["data"] = params
        data = eval("requests.%s" % methed)(_url, **kwargs)
    else:
        data = eval("requests.%s" % methed)(_url, params, **kwargs)

    r = data.json()

    # if r.status_code == 200:
    #     print(json.loads(r.content, encoding='utf-8'))
    # else:
    #     print(r.status_code, json.loads(r.content, encoding='utf-8'))

    cprint(r)
    return r

def get(url, params, **kwargs):
    admin_login_and_send("get", url, params, **kwargs)

def post(url, params, **kwargs):
    admin_login_and_send("post", url, params, **kwargs)

def put(url, params, **kwargs):
    admin_login_and_send("put", url, params, **kwargs)

def delete(url, params, **kwargs):
    admin_login_and_send("delete", url, params, **kwargs)




import queue
import threading
import logging
from traceback import format_exc

# global
thread_queue = None

class ThreadQueue(threading.Thread):
    def __init__(self, task_name, thread_size=10, queue_size=10):
        """
        多线性队列实现，FIFO先进先出
        初始化线程，并创建队列中间件queue
        thread_size：生成的线程个数
        queue_size：指定队列最大容量，当容量满时，队列将阻塞直到线程消费其中的任务

        注意：任务完成后，必须显式调用queue.join()结束线程
        :param thread_size:
        :param queue_size:
        """
        threading.Thread.__init__(self)

        self.task_name = task_name
        self.thread_size = thread_size
        global thread_queue
        if thread_queue is None:
            thread_queue = queue.Queue(queue_size)

        self.queue = thread_queue

    def run(self):
        """
        从队列里获取任务并执行
        :return:
        """
        while True:
            try:
                item = self.queue.get()  # 阻塞获取任务
                self.task_name(item)
                self.queue.task_done()   # task_name 执行完毕后，向队列发送一个信号
            except Exception as e:
                logging.error(format_exc())
                logging.error(e)
            finally:
                pass


    @classmethod
    def create_thread(cls, task_name, thread_size, queue_size):
        """
        # 返回生产者队列，只需要把任务task对象put进去即可。
        # 注意：当task含有数据库操作时，由于数据库exectuce带线程锁，资源互斥，必须确保每个task里用独立的数据库连接
        # 所有任务放进队列后，调用queue.join()挂起主线程，直到队列内所有任务完成，主线程结束，关闭所有子线程
        :param thread_size: 同时处理任务的线程数
        :param queue_size: 队列最大数量，建议设置thread_size*2，注意OOM
        :return:
        """
        for i in range(thread_size):
            t = ThreadQueue(task_name, thread_size, queue_size)
            t.setDaemon(True)
            t.start()

        return thread_queue



if __name__ == '__main__':
    # 定义任务
    def task_name(item):
        print(threading.current_thread().name)
        print(item)

    # 创建队列
    queue = ThreadQueue.create_thread(task_name, 10, 20)

    # 往队列中填数据
    for num in range(20):
        queue.put(num)

    # TODO wait on the queue until everything has been processed
    # 结束工作
    queue.join()
    print("done")





#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.permission_schema import PermissionClassSchema, PermissionGroupSchema
import copy, xlrd

class PermissionModel(BaseModel, PermissionGroupSchema):
    """
        权限
        Example:

            ::: assert self.model.PermissionModel.has_permission(gid, class_key, *[class_bit])

            1、******，******：
            assert self.model.PermissionModel.has_permission(account_info.get('gid'), 9, *[10, 11])
            2、******，******
            assert self.model.PermissionModel.has_permissions(account_info.get('gid'), [(9, 10), (3, 2), (3, 4)])

    """

    @classmethod
    def has_permission(cls, gid, class_key, *bits):
        """
        ******
        :param gid:
        :param class_key:
        :param bits: [1, 2, ..] ******
        :return:
        """
        permission = 0

        if not isinstance(bits, list):
            bits = list(bits)

        for bit in bits:
            permission += 2 ** int(bit)

        if not gid or not cls.Permission.check_permission(gid, class_key, permission):
            raise cls.api_error.ErrorNoApiPermission(u"******")
        else:
            return True

    @classmethod
    def is_has_permission(cls, gid, class_key, *bits):
        """
        ******

        :param gid:
        :param class_key:
        :param bits: [1, 2, ..] ******
        :return: True or False
        """
        permission = 0

        if not isinstance(bits, list):
            bits = list(bits)

        for bit in bits:
            permission += 2 ** int(bit)

        if not gid or not cls.Permission.check_permission(gid, class_key, permission):
            return False
        else:
            return True

    @classmethod
    def has_permissions(cls, gid, class_list):
        """
        ******
        :param gid:
        :param class_list: [(class_key, bit),(class_key, bit)]
        :return:
        """
        exception_num = 0
        for i in class_list:
            class_key, bit = i
            try:
                cls.has_permission(gid, class_key, bit)
            except cls.api_error.ErrorNoApiPermission as e:
                exception_num += 1
                continue
        if len(class_list) == exception_num:
            raise cls.api_error.ErrorNoApiPermission
        else:
            return True

    @classmethod
    def get_permission(cls, gid):
        """******"""

        me_group = {}

        permission_class = copy.deepcopy(cls.Permission.CURRENT_PERMISSION_CLASS)
        permission_group = copy.deepcopy(cls.Permission.CURRENT_PERMISSION_GROUP['group'])

        if gid not in permission_group:
            raise cls.api_error.ErrorInvalid("******")

        for group in cls.bit2mp(permission_group, permission_class):
            if int(gid) != group["gid"]:
                continue

            me_group = group
            break

        # TODO ******“1”******
        me_group["permission_class"] = me_group["permission_class"]
        return me_group

    @classmethod
    def get_current_permission(cls):
        """
        ******
        :param gid:
        :param pid:
        :return:
        """

        permission_class = copy.deepcopy(cls.Permission.CURRENT_PERMISSION_CLASS)
        permission_group = copy.deepcopy(cls.Permission.CURRENT_PERMISSION_GROUP['group'])

        d = {"current_permission": cls.bit2mp(permission_group, permission_class),
             "permission_group": cls.get_groups(copy.deepcopy(permission_group).values()),
             "permission_class": copy.deepcopy(permission_class)
             }

        # TODO ******“1”******
        for k, value in d.items():
            if k == "current_permission":
                for item in value:
                    item["bit_mp"] = item["bit_mp"]

            if k == "permission_class":
                d[k] = value[1]
                d[k]["bit_mp"] = sorted(d[k]["bit_mp"].values(), key=lambda o: o["bit"])
        return d

    @classmethod
    def bit2mp(cls, permission_group, permission_class):
        """
        ******
        ******
        {
            u'available': 1,
            u'name': u'xxx',
            u'pid': 1,
            u'create_at': 1513590899,
            u'gid': 1
            u'permission_class': {"1": {"0": 1, "1": 0, ...},
                                  "2": {"0": 1, "1": 0, ...},
                                  ...
                                },
        }
        """
        new_permission_group = []

        for gid, group in permission_group.items():

            tmp_permission_class = []
            for k, mask in group.get("permission_class", {}).items():

                # TODO ******1
                if int(k) > 1:
                    continue

                tmp_bit_mp = []
                for bit, item in permission_class[int(k)]["bit_mp"].items():
                    item["class_key"] = k
                    item["bit"] = int(bit)
                    # item["id"] = "%s-%s" % (k.encode("u8"), str(bit))

                    # 1 ****** 0 ******
                    item["state"] = 1 if int(mask & 1 << int(bit)) else 0
                    # bit_mp[str(bit)] = item
                    tmp_bit_mp.append(item)

                # tmp_permission_class[str(k)] = sorted(tmp_bit_mp, key=lambda o: o["bit"])
                tmp_permission_class = sorted(tmp_bit_mp, key=lambda o: o["bit"])

            group["bit_mp"] = copy.deepcopy(tmp_permission_class)
            new_permission_group.append(group)

        return sorted(new_permission_group, key=lambda o: o["gid"])

    @classmethod
    def get_groups(cls, permission_group):
        """
        ******
        :param permission_group:
        :return:
        """
        group_list = []
        for group in permission_group:
            group.pop("permission_class", True)
            group_list.append(group)

        group_list = sorted(group_list, key=lambda o: o["gid"])

        return group_list

    @classmethod
    def update_permission(cls, **permission_mp):
        """
        ******
        :param permission_mp:{"1": {"1": {"0": 1, "1": 1}, "2": ..}, "2": {..}}
        :return:
        """
        cls.Permission.update_batch_permission(**permission_mp)
        return cls.get_current_permission()

    @classmethod
    def create_group(cls, **group):
        """
        ******
        :param group:
                    {
                        "_id" : "5a379073cbc86e7124eb2c6c",
                        "available" : true,
                        "permission_class" : {
                            "1" : NumberLong(9223372036854775807),
                            "2" : NumberLong(9223372036854775807)
                        },
                        "name" : "xxx",
                        "pid" : 1,
                        "create_at" : 1513590899,
                        "gid" : 1
                    }
        :return:
        """
        return cls.Permission.create_group(**group)

    @classmethod
    def update_group_name(cls, gid, new_name):
        """
        ******,******
        :param group:
        :return:
        """

        d = cls.Permission.update_group_name(gid, new_name)
        if d is None:
            raise cls.api_error.ErrorGroup("******")
        return d

    @classmethod
    def create_class(cls, nums, **classes):
        """
        ******
        :param nums:  ******
        :param classes: {
                            "_id" : "5a379072cbc86e7124eb2c6a",
                            "available" : true,
                            "name" : "******:1",
                            "key" : 1,
                            "bit_mp": {..}
                        }
        :return:
        """

        return cls.Permission.create_class(nums, classes)

    @classmethod
    def create_bit(cls, class_key, name):
        """
        ******
        :param class_key:
        :param name:
        :return:
        """
        return cls.Permission.create_bit(class_key, name)

    @classmethod
    def update_class_name(cls, class_key, new_name):
        """
        ******
        :param group:
        :return:
        """

        d = cls.Permission.update_class_name(class_key, new_name)
        if d is None:
            raise cls.api_error.ErrorGroup("******")
        return d

    @classmethod
    def update_bit_name(cls, class_key, bit, new_nam):
        """
        ******
        :param group:
        :return:
        """

        d = cls.Permission.update_bit_name(class_key, bit, new_nam)
        if d is None:
            raise cls.api_error.ErrorGroup("******")
        return d

    @classmethod
    def update_setting(cls, dir):
        """
        ******，******
        :param dir:
        :return:
        """
        import os
        if not os.path.exists(dir):
            raise Exception("not exists %s" % dir)

        with xlrd.open_workbook(dir) as data:
            def __get_bit_mp(s, e):
                """******"""
                bit_mp = {}
                for i, name in enumerate(table.col_values(1)[s: e]):
                    bit_mp[str(i)] = {
                        "name": name,
                        "available": True,
                        "sort": i + 1,
                        "p_type": table.col_values(3 - 1)[s + i],  # ******
                        "desc": table.col_values(4 - 1)[s + i],  # ******
                        "path": table.col_values(5 - 1)[s + i],  # ******
                    }

                return bit_mp

            def __get_permission_vals(group_index, s, e):
                """******mask
                skip: ******，******
                """
                mask = 0
                skip = 5
                for i, v in enumerate(table.col_values(group_index + skip)[s: e]):
                    if not v and v != 0:
                        v = 0
                    mask += 2 ** i if int(v) else 0
                return mask

            # ******
            table = data.sheets()[0]
            # ******
            cell_values = table._cell_values

            permission_group = []  # ******
            permission_class = []  # ******

            # ******
            groups_list = [n for n in cell_values[1][5:] if n]
            start_gid = 1000
            for i, name in enumerate(groups_list):
                if i == 0:
                    pid = 0
                else:
                    pid = i + start_gid - 1
                permission_group.append({
                    "_id": cls.utils.ObjectId_to(ObjectId()),
                    "name": name,
                    "gid": i + start_gid,
                    "pid": pid,
                    "available": 1,
                    "desc": "",
                    "permission_class": {},
                    "create_at": cls.utils.now()
                })

            # ******
            index_s_e = ([3, 9], [10, 15], [16, 34], [35, 40], [41, 55], [56, 76], [77, 79], [80, 99], [100, 116])
            key = 1
            for s, e in index_s_e:
                s, e = s - 1, e

                for class_name in table.col_values(0)[s: e]:
                    if not class_name:
                        continue

                    # ******
                    permission_class.append({
                        "_id": str(cls.utils.ObjectId()),
                        "name": class_name,
                        "key": str(key),
                        "desc": "",
                        "available": 1,
                        "bit_mp": __get_bit_mp(s, e),
                        "create_at": cls.utils.now()
                    })

                    # ******
                    for i, group in enumerate(permission_group):
                        group["permission_class"][str(key)] = __get_permission_vals(i, s, e)

                    key += 1
                    break

        # assert cls.Permission.check_permission_group(permission_group) is True
        # assert cls.Permission.check_permission_class(permission_class) is True
        # cls.mongo[cls.Permission.PERMISSION_TABLE].remove({}, multi=True)
        cls.mongo[cls.Permission.PERMISSION_CLASS_TABLE].remove({}, multi=True)
        # cls.mongo[cls.Permission.PERMISSION_TABLE].insert(permission_group)
        cls.mongo[cls.Permission.PERMISSION_CLASS_TABLE].insert(permission_class)
        cls.Permission.reload()

    @classmethod
    def get_parent_group(cls, gid, multi=False):
        """
        ******
        :param gid:
        :return:multi=True ******
        """
        if multi:
            return cls.Permission.get_parent_groups(int(gid))
        else:
            return cls.Permission.get_parent_group(int(gid))

    @classmethod
    def get_child_group(cls, gid, multi=False):
        """
        ******
        :param gid:
        :return:multi=True ******
        """
        if multi:
            return cls.Permission.get_child_groups(int(gid))
        else:
            return cls.Permission.get_child_group(int(gid))

    @classmethod
    def get_permission_group(cls, gid):

        """
        ******
        :param gid:
        :return:multi=True ******
        """
        return cls.Permission.get_permission_group(int(gid))

    @classmethod
    def get_childes(cls, gid, multi=False, fields=False):

        """
        ******
        :param gid: ******ID
        :param multi: ******
        :param fields: ******gid******name
        :return:
        """
        child_role_list = cls.get_child_group(gid, multi=multi)
        if fields:
            return [{'gid': i.get('gid'), 'name': i.get('name')} for i in child_role_list]
        else:
            return [i.get('gid') for i in child_role_list]

    @classmethod
    def get_parents(cls, gid, multi=False, fields=False):

        """
        ******
        :param gid: ******ID
        :param multi: ******
        :param fields: ******gid******name
        :return:
        """
        parent_role_list = cls.get_parent_group(gid, multi=multi)
        if fields:
            return [{'gid': i.get('gid'), 'name': i.get('name')} for i in parent_role_list]
        else:
            return [i.get('gid') for i in parent_role_list]

    @classmethod
    def get_permission_by_spec(cls, spec, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        d = cls.find_one(spec)
        return cls._parse_data(d)

    @classmethod
    def get_permission_list_by_last_id(cls, last_id, count, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        spec = {}.update(kwargs.get("spec", {}))
        data_list, last_id = cls.find_list_by_last_id(spec,
                                                      count,
                                                      sort=[("create_at", -1), ],
                                                      fields=False,
                                                      last_id_field=False)
        return [cls._parse_data(d) for d in data_list if d], last_id

    @classmethod
    def get_permission_list_by_page(cls, page, count, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, pages = cls.find_list_by_page(spec,
                                                 page,
                                                 count,
                                                 sort=[("create_at", -1), ],
                                                 fields=False)
        return [cls._parse_data(d) for d in data_list if d], pages

    @classmethod
    def create_permission(cls, *args, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        pass

    @classmethod
    def modify_permission(cls, *args, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        pass

    @classmethod
    def delete_permission(cls, permission_id, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        return cls.update({"_id": permission_id}, {"state": -1})

    @classmethod
    def _parse_data(cls, d, **kwargs):
        """
        ******
        :return:
        :rtype:
        """
        if not d:
            return {}

        d["permission_id"] = d["_id"]

        return d



